#######  This script accompanies the manuscript published in Ecosystem Services:
######  Branoff,Benjamin L., Cicchetti, G., Jackson, SJ, Pryor, M., Sharpe, L.M., Shumchenia, E., and and Susan H. Yee. (2023) A Matrix Approach to Quantify Spatio-temporal Trends in Ecosystem Services Capacity for Massachusetts Bays National Estuary Embayments

##  this script helps organize and aggregate the individual matrices from the MassBays regional coordinators
##  into one single matrix that can be used in the analysis

library(readxl)
library(plyr)

files <- list.files(getwd(),pattern=".xlsx")
for (f in files){
  if(grepl("LowerNorth",f)){loc <- "Lower North Shore" }
  else if (grepl("SaraGrady",f)){loc <- "South Shore" }
  else if (grepl("UpperNorth",f)){loc <- "Upper North Shore" }
  else if (grepl("Cape Cod",f)){loc <- "Cape Cod" }
  else if (grepl("Boston",f)){loc <- "Boston" }
  sheets <- readxl::excel_sheets(f)
 # sheets <- sheets[sheets!="Focal"]
  for (s in sheets[[1]]){
    sht <- data.frame(read_excel(f, sheet = s))
    names(sht)[1] <- "EcoSystem"
    if (s==sheets[1]){sheet <- sht}else{sheet=merge(sheet,sht,by="EcoSystem",all=T)}
  }
  sheet$source <- basename(f)
  sheet$Region <- loc
  if (f==files[1]){Sheets <- sheet}else{Sheets <- rbind.fill(Sheets,sheet)}
}
Ec <- c("Evergreen Forest","Deciduous Forest","Mixed Forest",
        "Scrub/Shrub","Grassland/Herbaceous","High Estuarine Emergent Wetland",
        "Low Estuarine Emergent Wetland","Estuarine Aquatic Bed",
        "Unconsolidated Shore","Benthos","Diadromous Fish runs",
        "Palustrine Emergent Wetland","Palustrine Forested Wetland",
        "Open Water","Pasture/Hay","Cultivated Crops",
        "Barren Land","Developed, Open Space","Urban")
ES <- c("Water.movement","Water.quality","Water.quantity", 
        "Charismatic.fauna","Commercially.important.fauna","Edible.fauna", "Fauna.community",
        "Keystone.fauna","Medicinal.fauna","Pest.predator.depredator.fauna","Rare.fauna","Spiritually.culturally.important.fauna",
        "Protection.From.Extreme.weather.events", "Protection.From.Flooding", "Protection.From.Wildfire","Ecological.condition",                                                                             
        "Open.space", "Phenomena..e.g...sunsets..northern.lights..etc.","Scents","Sounds", "Viewscapes",                                                                                       
        "Charismatic.flora", "Commercially.important.flora", "Edible.flora", "Flora.community", "Keystone.flora",
        "Medicinal.flora","Rare.flora", "Spiritually.culturally.important.flora",
        "Commercially.important.fungi", "Edible.fungi","Fungi.community","Keystone.fungi","Medicinal.fungi",
        "Rare.fungi", "Spiritually.culturally.important.fungi",
        "Air.quality", "Precipitation","Sunlight","Temperature","Wind.strength.speed",                                                                              
        "Soil.quality","Soil.quantity","Substrate.quality","Substrate.quantity",
        "Fiber.material.quality","Fiber.material.quantity","Fuel.quality","Fuel.quantity",
        "Mineral.chemical.quality","Mineral.chemical.quantity",
        "Presence.of.other.natural..materials.for.artistic.use.or.consumption..e.g..shells..acorns..honey.",
        "Climate.change.impacts.mitigation",
        "Recreation..swimming..boating..fishing..shellfishing..tourism.",
        "Aesthetic.and.economic.value")
Sheets$EcoSystem[which(!Sheets$EcoSystem %in% Ec)]
names(Sheets)[which(!names(Sheets) %in% ES)]

sheetsL <- reshape(Sheets,direction="long",idvar=c("EcoSystem","source","Region"),times=names(Sheets)[which(!names(Sheets) %in% c("EcoSystem","source","Region"))],
                   varying=list(names(Sheets)[which(!names(Sheets) %in% c("EcoSystem","source","Region"))]),timevar = "ES")
names(sheetsL)[5] <- "Value"
sheetsL$ES2 <- sheetsL$ES
sheetsL$ES2[grepl("Extreme|Flooding|resiliency",sheetsL$ES)] <- "Protection.From.Extreme.weather.events.and.Flooding"
sheetsL$ES2[grepl("Edible.fauna|Commercially.important.fauna|Food",sheetsL$ES)] <- "Edible.or.Commercially.important.fauna"
sheetsL$ES2[grepl("Fauna.community",sheetsL$ES)] <- "Fauna.community"
sheetsL$ES2[grepl("Water.quality",sheetsL$ES)] <- "Water.quality"
sheetsL$ES2[grepl("Water.quantity",sheetsL$ES)] <- "Water.quantity"
sheetsL <- sheetsL[sheetsL$ES2 %in% c("Protection.From.Extreme.weather.events.and.Flooding","Edible.or.Commercially.important.fauna",
                                      "Fauna.community","Water.quality","Water.quantity",'Climate.change.impacts.mitigation'),]
sheetsL <- sheetsL[sheetsL$EcoSystem %in% c("Estuarine Aquatic Bed","High Estuarine Emergent Wetland","Low Estuarine Emergent Wetland",
                                            "Unconsolidated Shore"),]
sheetsL$Value <- as.numeric(sheetsL$Value)
SheetsAgg <- do.call(data.frame,aggregate(data=sheetsL,Value~EcoSystem+ES2,FUN=function(x) c(mean=mean(x,na.rm=T),sd=sd(x,na.rm=T))))
SheetsAgg$Region <- "All"
names(SheetsAgg)[names(SheetsAgg)=="Value.mean"] <- "Value"
sheetsL$Value.sd <- NA
SheetsAgg <- rbind(SheetsAgg,sheetsL[,names(SheetsAgg)])
write.csv(SheetsAgg,"RC_MasterAgg.csv",row.names=F)


Master_aggLit <- read.csv("C:/Users/bbranoff/OneDrive - Environmental Protection Agency (EPA)/Profile/Documents/Ecosystem Services/Matrices/Master_agg_PII.csv")
Master_aggLit$NESCS_PII <- gsub(" ",".",Master_aggLit$NESCS_PII)
Master_aggLit$NESCS_PII[Master_aggLit$NESCS_PII=="Commerically.important.fauna"] <- "Edible.or.Commercially.important.fauna"
unique(Master_aggLit$NESCS_PII)[which(!unique(Master_aggLit$NESCS_PII) %in% unique(SheetsAgg$ES2))]
