###  a set of helper functions to process data for the manuscript:
######  Branoff,Benjamin L., Cicchetti, G., Jackson, SJ, Pryor, M., Sharpe, L.M., Shumchenia, E., and and Susan H. Yee. (2023) A Matrix Approach to Quantify Spatio-temporal Trends in Ecosystem Services Capacity for Massachusetts Bays National Estuary Embayments
###  these functions allow for much more rapid processing of extermely large matrices
##  the functions are called by the "analysis" script to run the bootstrap of the ecoystems services change 



cppFunction("double josilberRcpp(NumericVector area_c, NumericVector names,NumericMatrix short_p) {
double y;
const int nv = names.length();
    for (int i=0; i < nv;i++) {
      for (int j=0; j < nv; j++) {
        y+=(area_c(names(i)) * area_c(names(j)))/(1 + short_p(i, j));
      }
    }
   return y;
}")


cppFunction('NumericMatrix mmult(const NumericMatrix& m1,const double mdist){
NumericMatrix out(m1.nrow()*m1.ncol()/2,2);
int count = 0;
for (int i = 0; i < m1.nrow(); ++i) {
    for (int j = 0; j < m1.ncol(); ++j) {
    if (i < j){
      if (m1(i,j) > 0){
        if (m1(i,j) < mdist){
          out(count,0) = i+1;
          out(count,1) = j+1;
          count += 1;
        }
      }
    }
  }
}
return out;
}')

# create rnorm_cpp function ----
cppFunction('NumericVector rnorm_cpp(int n, double mean, double sd) { 
return Rcpp::rnorm(n, mean, sd); }')

cppFunction('NumericVector ESDist(NumericVector a, NumericVector sc, NumericVector sd, NumericVector conn){
int n = a.size();
int l = 1;
NumericVector scs(l);
NumericVector out(n);
for (int i = 0; i < n; ++i) {
    l = a[i]/900;
    if (R_IsNA(sd[i])){sd[i]=0;}
    if (l > 0){
    scs = Rcpp::rnorm(l,sc[i],sd[i]);
    out[i] = sum((900*scs)+(900*scs*conn[i]));
    }else{
    out[i] = 0;
    }
}
return out;
}')



# without using the lconnect functions, can do this
IIC_Calc1 <- function(land,max_dist=1,clean=F,clustered=F,scale=NA,metric=NA){
  distance <- st_distance(land)
  if (nrow(land)==1){
    clusters=1
  }else{
    groups <- hclust(as.dist(distance), "single")
    clusters <- cutree(groups, h = max_dist)
  }
  if (clustered==T){
    for (g in unique(clusters)){
      land1 <- land[which(clusters==g),]
      if (length(clusters[which(clusters==g)])>1){
        boundary <- st_convex_hull(st_union(land1))
        area_l <- st_area(boundary)
        distance1 <- st_distance(land1)
        if (metric=="IIC"){
          met_T <- as.numeric(IIC_calc2(land1,distance1,max_dist=500,area_l=area_l,clean=clean))
        } else if (metric=="PC"){
          met_T <- as.numeric(PCcalc(land1,area_l=area_l,clusters=clusters))
        }
        gc()
        npatch <- nrow(land1)
        dconn <- met <- rep(NA, npatch)
        conn <- 100;i=1;
        while(as.numeric(conn)>1&i<=npatch){#    for (i in 1:nrow(land)){
          land2 <- land1[-i,]
          dist <- distance1[-i,-i]
          clusters = clusters[-i]
          if (metric=="IIC"){
            met[i] <- as.numeric(IIC_calc2(land2,dist,max_dist=500,area_l=area_l,clean=clean))
          } else if (metric=="PC"){
            met[i] <- as.numeric(PCcalc(land2,area_l=area_l,clusters=clusters))
          }
          conn <- 100 * ((met_T - met[i])/met_T)
          if (is.nan(conn)){conn=0}
          dconn[i] <- conn
          i=i+1
        }
        Connections <<- rbind(Connections,data.frame(patchID =land1$patchID,cluster=g,ng=npatch,n=nrow(land),Class=L,year=y,scale=scale,Variable="Index_inv.",Value=met,met=metric),
                             data.frame(patchID =land1$patchID,cluster=g,ng=nrow(land1),n=nrow(land),Class=L,year=y,scale=scale,Variable="Conn.",Value=dconn,met=metric),
                             data.frame(patchID ="all",cluster=g,ng=nrow(land1),n=nrow(land),Class=L,year=y,scale=scale,Variable="Index",Value=met_T,met=metric)) 
        Connections <<- Connections[!is.na(Connections$Value),]
      }
    }
  }else{
    boundary <- st_convex_hull(st_union(land))
    area_l <- st_area(boundary)
    if (metric=="IIC"){
      met_T <- as.numeric(IIC_calc2(land,distance,max_dist,area_l=area_l,clean=clean))
    } else if (metric=="PC"){
      met_T <- as.numeric(PCcalc(land,area_l=area_l,clusters=clusters))
    }
    gc()
    npatch <- nrow(land)
    if (unique(land$Class) %in% c(18.0,18.1,18.2,18.3,19.0,23.0,24)&npatch>1){
      dconn <- met <- rep(NA, npatch)
      conn <- 100;i=1;
      while(as.numeric(conn)>1&i<=npatch){#    for (i in 1:nrow(land)){
        land2 <- land[-i,]
        dist <- distance[-i,-i]
        clusters2 = clusters[-i]
        if (metric=="IIC"){
          met[i] <- as.numeric(IIC_calc2(land2,dist,max_dist=500,area_l=area_l,clean=clean))
        } else if (metric=="PC"){
          met[i] <- as.numeric(PCcalc(land2,area_l=area_l,clusters=clusters2))
        }
        conn <- 100 * ((met_T - met[i])/met_T)
        if (is.nan(conn)){conn=0}
        dconn[i] <- conn
        i=i+1
      }
    }else{
      Conn <- NA;met<-NA;dconn<-NA
    }
    Connections <<- rbind(Connections,data.frame(patchID =land$patchID,cluster=NA,n=npatch,Class=L,year=y,scale=scale,Variable="Index_inv.",Value=met,met=metric),
                          data.frame(patchID =land$patchID,cluster=NA,n=npatch,Class=L,year=y,scale=scale,Variable="Conn.",Value=dconn,met=metric),
                          data.frame(patchID ="all",cluster=NA,n=npatch,Class=L,year=y,scale=scale,Variable="Index",Value=met_T,met=metric)) 
    Connections <<- Connections[!is.na(Connections$Value),]
  }
}
IIC_calc2 <- function(land,distance,max_dist,area_l,clean){
  area_c <- st_area(land)
  if (clean==T){gc()}
  distance <- stats::as.dist(distance)
  distance <- as.matrix(distance)
  class(distance) <- "numeric"
  e1 <- as.data.frame(mmult(distance,max_dist))
  e1 <- e1[which(e1[,1]>0),]
  e1 <- e1[order(e1[,2],e1[,1]),]
  #return(e1)
  g1 <- igraph::graph_from_data_frame(e1, directed = FALSE)
  short_p <- igraph::shortest.paths(g1)
  #short_p[!is.finite(short_p)] <- 0
  out <- josilberRcpp(area_c,as.numeric(row.names(short_p))-1,short_p)
  result <- out/(area_l^2)
  result
}

PCcalc <- function(land,area_l,clusters) {
  NC <- max(clusters)
  area_c <- st_area(land)
  df0 <- data.frame(area_c, clusters)
  r0 <- rep(NA, NC)
  for (i in 1:NC) {
    df1 <- df0[df0$clusters == i, ]
    ci <- sum(df1[, 1])
    r0[i] <- (ci/area_l)^2
  }
  sum(r0)
}

calc_dist <- function(land){
  y <- unique(land$year)
  files <- list.files("./Data/Intermediate/",pattern=".bk",full.names = T)
    if (sum(grepl(y,files))>0){
      Indices <- read.csv(paste0("./Data/Intermediate/Indices_",y,".csv"))
      distance <- as.dist(st_distance(land))
      distance <- as_bigdist(distance,file=file.path(tempdir(), "tempfile"))
      distance <- bigdist(file=file.path(paste0(tempdir(),"/tempfile_",nrow(land),"_double")))
      distance <- bigdist_extract(distance)
      big_d <- bigdist(file=gsub(".bk","",files[grep(y,files)]))
      big_d <- bigdist_extract(big_d)
      big_d <- cbind(big_d,matrix(NA,nrow=nrow(big_d),ncol=ncol(distance)))
      big_d <- rbind(big_d,matrix(NA,nrow=nrow(distance),ncol=ncol(big_d)))
      big_d[(max(Indices$index)+1):nrow(big_d),(max(Indices$index)+1):ncol(big_d)] <- distance
      as_bigdist(as.dist(big_d),file=paste0("./Data/Intermediate/",y))
      file.remove(files[grep(y,files)])
      file.remove(file.path(paste0(tempdir(),"/tempfile_",nrow(land),"_double.bk")))
      Indices <- rbind(Indices,
                       data.frame(index=(max(Indices$index)+1):(max(Indices$index)+nrow(land)),
                                  patchID=land$patchID,Class=land$Class))
    }else{
      dist <- as.dist(st_distance(land))
      as_bigdist(dist,file=paste0("./Data/Intermediate/",y))
      Indices <- data.frame(index=1:nrow(land),patchID=land$patchID,Class=land$Class)
      write.csv(Indices,paste0("./Data/Intermediate/Indices_",y,".csv"),row.names = F)
    }
  dist
}

find_dist <- function(land){
  y <- unique(land$year)
  pairs <- expand.grid(unique(land$Class), unique(land$Class))
  pairs <- pairs[-which(pairs$Var1==pairs$Var2),]
  files <- list.files("./Data/Intermediate/",pattern=".bk",full.names = T)
  Indices <- read.csv(paste0("./Data/Intermediate/Indices_",y,".csv"))
  for (p in 1:nrow(pairs)){
    
  }
  big_d <- bigdist(file=gsub(".bk","",files[grep(y,files)]))
  big_d <- bigdist_extract(big_d)
  big_d <- big_d[Indices[Indices$patchID %in% land$patchID],Indices[Indices$patchID %in% land$patchID]]
}
